@@ -64,8 +64,6 @@ from wsgiref.headers import Headers
 
 import gevent
 from gevent.pywsgi import WSGIServer
-from dns import reversename
-from dns.exception import DNSException
 
 from slimta.logging import log_exception
 from slimta.http.wsgi import WsgiServer
@@ -73,7 +71,7 @@ from slimta.envelope import Envelope
 from slimta.smtp.reply import Reply
 from slimta.queue import QueueError
 from slimta.relay import RelayError
-from slimta.util import dns_resolver
+from slimta.util.ptrlookup import PtrLookup
 from . import Edge
 
 __all__ = ['WsgiResponse', 'WsgiEdge', 'WsgiValidators']
@@ -108,7 +106,7 @@ def _build_http_response(smtp_reply):
         info['command'] = smtp_reply.command
     Headers(headers).add_header('X-Smtp-Reply', code, **info)
     if code.startswith('2'):
-        return WsgiResponse('200 OK', headers)
+        return WsgiResponse('204 No Content', headers)
     elif code.startswith('4'):
         return WsgiResponse('503 Service Unavailable', headers)
     elif code == '535':
@@ -158,17 +156,18 @@ class WsgiEdge(Edge, WsgiServer):
                  uri_pattern=None):
         super(WsgiEdge, self).__init__(queue, hostname)
         self.validator_class = validator_class
-        if isinstance(uri_pattern, basestring):
+        if isinstance(uri_pattern, str):
             self.uri_pattern = re.compile(uri_pattern)
         else:
             self.uri_pattern = uri_pattern
 
     def __call__(self, environ, start_response):
-        self._trigger_ptr_lookup(environ)
+        ptr_lookup = PtrLookup(environ.get('REMOTE_ADDR', '0.0.0.0'))
+        ptr_lookup.start()
         try:
             self._validate_request(environ)
             env = self._get_envelope(environ)
-            self._add_envelope_extras(environ, env)
+            self._add_envelope_extras(environ, env, ptr_lookup)
             self._enqueue_envelope(env)
         except WsgiResponse as res:
             start_response(res.status, res.headers)
@@ -176,12 +175,13 @@ class WsgiEdge(Edge, WsgiServer):
         except Exception as exc:
             log_exception(__name__)
             msg = '{0!s}\n'.format(exc)
-            headers = [('Content-Length', len(msg)),
+            content_length = str(len(msg))
+            headers = [('Content-Length', content_length),
                        ('Content-Type', 'text/plain')]
             start_response('500 Internal Server Error', headers)
             return [msg]
         finally:
-            environ['slimta.ptr_lookup_thread'].kill(block=False)
+            ptr_lookup.kill(block=False)
 
     def _validate_request(self, environ):
         if self.uri_pattern:
@@ -209,28 +209,12 @@ class WsgiEdge(Edge, WsgiServer):
             cgi_name = _header_name_to_cgi(name)
             validators.validate_custom(name, environ.get(cgi_name))
 
-    def _ptr_lookup(self, environ):
-        ip = environ.get('REMOTE_ADDR', '0.0.0.0')
-        try:
-            ptraddr = reversename.from_address(ip)
-            try:
-                answers = dns_resolver.query(ptraddr, 'PTR')
-            except NXDOMAIN:
-                answers = []
-            try:
-                environ['slimta.reverse_address'] = str(answers[0])
-            except IndexError:
-                pass
-        except Exception:
-            log_exception(__name__, query=ip)
-
-    def _trigger_ptr_lookup(self, environ):
-        thread = gevent.spawn(self._ptr_lookup, environ)
-        environ['slimta.ptr_lookup_thread'] = thread
+    def _b64decode(self, b64str):
+        return b64decode(b64str.encode('ascii')).decode('utf-8')
 
     def _get_sender(self, environ):
         sender_header = _header_name_to_cgi(self.sender_header)
-        return b64decode(environ.get(sender_header, ''))
+        return self._b64decode(environ.get(sender_header, ''))
 
     def _get_recipients(self, environ):
         rcpt_header = _header_name_to_cgi(self.rcpt_header)
@@ -238,7 +222,7 @@ class WsgiEdge(Edge, WsgiServer):
         if not rcpts_raw:
             return []
         rcpts_split = self.split_pattern.split(rcpts_raw)
-        return [b64decode(rcpt_b64) for rcpt_b64 in rcpts_split]
+        return [self._b64decode(rcpt_b64) for rcpt_b64 in rcpts_split]
 
     def _get_ehlo(self, environ):
         ehlo_header = _header_name_to_cgi(self.ehlo_header)
@@ -255,16 +239,17 @@ class WsgiEdge(Edge, WsgiServer):
         env.parse(data)
         return env
 
-    def _add_envelope_extras(self, environ, env):
+    def _add_envelope_extras(self, environ, env, ptr_lookup):
         env.client['ip'] = environ.get('REMOTE_ADDR', 'unknown')
-        env.client['host'] = environ.get('slimta.reverse_address', None)
+        env.client['host'] = ptr_lookup.finish()
         env.client['name'] = self._get_ehlo(environ)
         env.client['protocol'] = environ.get('wsgi.url_scheme', 'http').upper()
 
     def _enqueue_envelope(self, env):
         results = self.handoff(env)
         if isinstance(results[0][1], QueueError):
-            reply = Reply('550', '5.6.0 Error queuing message')
+            default_reply = Reply('451', '4.3.0 Error queuing message')
+            reply = getattr(results[0][1], 'reply', default_reply)
             raise _build_http_response(reply)
         elif isinstance(results[0][1], RelayError):
             relay_reply = results[0][1].reply
