@@ -36,7 +36,7 @@ class SocketLogger(object):
     use. This prevents free-form logs from mixing in with standard, machine-
     parseable logs.
 
-    :param log: :class:python:`logging.Logger` object to log through.
+    :param log: :py:class:`logging.Logger` object to log through.
 
     """
 
@@ -74,8 +74,9 @@ class SocketLogger(object):
 
         """
         client_peer = address or client.getpeername()
-        self.log(server.fileno(), 'accept', clientfd=client.fileno(),
-                                            peer=client_peer)
+        self.log(server.fileno(), 'accept',
+                 clientfd=client.fileno(),
+                 peer=client_peer)
 
     def connect(self, socket, address=None):
         """Logs a socket :meth:`~socket.socket.connect()` operation along with
@@ -89,8 +90,9 @@ class SocketLogger(object):
         self.log(socket.fileno(), 'connect', peer=peer)
 
     def encrypt(self, socket, tls_args):
-        """Logs a socket encryption operation along with the certificate and key
-        files used and whether the socket is acting as the client or the server.
+        """Logs a socket encryption operation along with the certificate and
+        key files used and whether the socket is acting as the client or the
+        server.
 
         :param socket: The socket that was shutdown.
         :param tls_args: Keyword rguments passed to the encryption operation.
@@ -99,8 +101,10 @@ class SocketLogger(object):
         keyfile = tls_args.get('keyfile', None)
         certfile = tls_args.get('certfile', None)
         server_side = tls_args.get('server_side', False)
-        self.log(socket.fileno(), 'encrypt', keyfile=keyfile, certfile=certfile,
-                                             server_side=server_side)
+        self.log(socket.fileno(), 'encrypt',
+                 keyfile=keyfile,
+                 certfile=certfile,
+                 server_side=server_side)
 
     def shutdown(self, socket, how):
         """Logs a socket :meth:`~socket.socket.shutdown()` operation along
@@ -127,5 +131,26 @@ class SocketLogger(object):
         """
         self.log(socket.fileno(), 'close')
 
+    def proxyproto_success(self, socket, src_addr):
+        """Logs a successful proxy protocol header, with the new source address
+        information. Logged at the ``DEBUG`` level.
+
+        :param socket: The socket that received the header.
+        :param src_addr: The new source address information.
+
+        """
+        self.log(socket.fileno(), 'ppsuccess', peer=src_addr)
+
+    def proxyproto_invalid(self, socket, exc):
+        """Logs an invalid proxy protocol header, with an exception provided by
+        the :class:`~slimta.util.proxyproto.ProxyProtocol` class. Logged at the
+        ``WARNING`` level.
+
+        :param socket: The socket that failed the proxy protocol.
+        :param exc: The exception generated during the failure.
+
+        """
+        self.log(socket.fileno(), 'ppinvalid', message=str(exc))
+
 
 # vim:et:fdm=marker:sts=4:sw=4:ts=4
