@@ -30,13 +30,16 @@ from __future__ import absolute_import
 
 import time
 import bisect
-from itertools import imap, izip, repeat, chain
+import collections
+from itertools import repeat, chain
 
 import gevent
 from gevent import Greenlet
 from gevent.event import Event
-from gevent.coros import Semaphore
+from gevent.lock import Semaphore
 from gevent.pool import Pool
+import six
+from six.moves import map
 
 from slimta.logging import log_exception
 from slimta.core import SlimtaError
@@ -49,7 +52,14 @@ __all__ = ['QueueError', 'Queue', 'QueueStorage']
 
 
 class QueueError(SlimtaError):
-    """Base exception for errors in the queue package."""
+    """Base exception for errors in the queue package.
+
+    .. attribute:: reply
+
+       If set, |Edge| services may use this as the |Reply| object to return to
+       clients when queuing fails.
+
+    """
     pass
 
 
@@ -61,6 +71,10 @@ class QueueStorage(object):
     def __init__(self):
         pass
 
+    def _remove_delivered_rcpts(self, envelope, rcpt_indexes):
+        for index in sorted(rcpt_indexes, reverse=True):
+            del envelope.recipients[index]
+
     def write(self, envelope, timestamp):
         """Writes the given envelope to storage, along with the timestamp of
         its next delivery attempt. The number of delivery attempts asociated
@@ -95,6 +109,22 @@ class QueueStorage(object):
         """
         raise NotImplementedError()
 
+    def set_recipients_delivered(self, id, rcpt_indexes):
+        """.. versionadded:: 1.1.0
+
+        Marks the given recipients from the original envelope as
+        already-delivered, meaning they will be skipped by future relay
+        attempts.
+
+        :param id: The unique identifier string for the message.
+        :param rcpt_indexes: List of indexes in the original envelope's
+                             :attr:`~slimta.envelope.Envelope.recipients` list
+                             to mark as delivered.
+        :raises: :class:`QueueError`
+
+        """
+        raise NotImplementedError()
+
     def load(self):
         """Loads all queued messages from the storage engine, such that the
         :class:`Queue` can be aware of all upcoming delivery attempts.
@@ -110,6 +140,9 @@ class QueueStorage(object):
         """Returns the |Envelope| object associated with the given unique
         identifier sring.
 
+        The envelope's :attr:`~slimta.envelope.Envelope.recipients` should not
+        include those marked as delivered by :meth:`.set_recipients_delivered`.
+
         :param id: The unique identifier string of the requested |Envelope|.
         :returns: Tuple with the |Envelope| object and number of attempts.
         :raises: :class:`KeyError`, :class:`QueueError`
@@ -134,15 +167,17 @@ class QueueStorage(object):
         is notified that a new message has been stored.
 
         :returns: An iterable or generator producing tuples with the timestamp
-                  and unique identifier string of a new message in storage. When
-                  the iterable or generator is exhausted, :meth:`.wait` is
+                  and unique identifier string of a new message in storage.
+                  When the iterable or generator is exhausted, :meth:`.wait` is
                   simply called again.
 
         """
         raise NotImplementedError()
 
     def get_info(self):
-        """Queries the storage backend for relevant information about the
+        """.. versionadded:: 0.3.20
+
+        Queries the storage backend for relevant information about the
         contents of the queue. The result is a :func:`dict` containing required
         keys along with any other custom keys dependent on the particular
         backend.
@@ -175,6 +210,8 @@ class Queue(Greenlet):
                            same parameters as the |Bounce| constructor. If the
                            function returns ``None``, no bounce is delivered.
                            By default, a new |Bounce| is created in every case.
+    :param bounce_queue: |Queue| object that will be used for delivering bounce
+                         messages. The default is ``self``.
     :param store_pool: Number of simultaneous operations performable against
                        the ``store`` object. Default is unlimited.
     :param relay_pool: Number of simultaneous operations performable against
@@ -183,12 +220,13 @@ class Queue(Greenlet):
     """
 
     def __init__(self, store, relay=None, backoff=None, bounce_factory=None,
-                 store_pool=None, relay_pool=None):
+                 bounce_queue=None, store_pool=None, relay_pool=None):
         super(Queue, self).__init__()
         self.store = store
         self.relay = relay
         self.backoff = backoff or self._default_backoff
         self.bounce_factory = bounce_factory or Bounce
+        self.bounce_queue = bounce_queue or self
         self.wake = Event()
         self.queued = []
         self.active_ids = set()
@@ -251,7 +289,7 @@ class Queue(Greenlet):
 
     def _pool_imap(self, which, func, *iterables):
         pool = getattr(self, which+'_pool', gevent)
-        threads = imap(pool.spawn, repeat(func), *iterables)
+        threads = map(pool.spawn, repeat(func), *iterables)
         ret = []
         for thread in threads:
             thread.join()
@@ -284,7 +322,7 @@ class Queue(Greenlet):
         envelopes = self._run_policies(envelope)
         ids = self._pool_imap('store', self.store.write, envelopes,
                               repeat(now))
-        results = zip(envelopes, ids)
+        results = list(six.moves.zip(envelopes, ids))
         for env, id in results:
             if not isinstance(id, BaseException):
                 if self.relay:
@@ -299,35 +337,54 @@ class Queue(Greenlet):
             self._add_queued(entry)
 
     def _remove(self, id):
-        self.store.remove(id)
+        self._pool_spawn('store', self.store.remove, id)
         self.queued_ids.discard(id)
         self.active_ids.discard(id)
 
     def _bounce(self, envelope, reply):
         bounce = self.bounce_factory(envelope, reply)
         if bounce:
-            return self.enqueue(bounce)
+            return self.bounce_queue.enqueue(bounce)
 
     def _perm_fail(self, id, envelope, reply):
-        self._pool_spawn('store', self._remove, id)
+        if id is not None:
+            self._remove(id)
         if envelope.sender:  # Can't bounce to null-sender.
             self._pool_spawn('bounce', self._bounce, envelope, reply)
 
-    def _retry_later(self, id, envelope, reply):
+    def _split_by_reply(self, envelope, replies):
+        if isinstance(replies, Reply):
+            return [(replies, envelope)]
+        groups = []
+        for i, rcpt in enumerate(envelope.recipients):
+            for reply, group_env in groups:
+                if replies[i] == reply:
+                    group_env.recipients.append(rcpt)
+                    break
+            else:
+                group_env = envelope.copy([rcpt])
+                groups.append((replies[i], group_env))
+        return groups
+
+    def _retry_later(self, id, envelope, replies):
         attempts = self.store.increment_attempts(id)
         wait = self.backoff(envelope, attempts)
         if wait is None:
-            reply.message += ' (Too many retries)'
-            self._perm_fail(id, envelope, reply)
+            for reply, group_env in self._split_by_reply(envelope, replies):
+                reply.message += ' (Too many retries)'
+                self._perm_fail(None, group_env, reply)
+            self._remove(id)
+            return False
         else:
             when = time.time() + wait
             self.store.set_timestamp(id, when)
             self.active_ids.discard(id)
             self._add_queued((when, id))
+            return True
 
     def _attempt(self, id, envelope, attempts):
         try:
-            self.relay._attempt(envelope, attempts)
+            results = self.relay._attempt(envelope, attempts)
         except TransientRelayError as e:
             self._pool_spawn('store', self._retry_later, id, envelope, e.reply)
         except PermanentRelayError as e:
@@ -338,7 +395,40 @@ class Queue(Greenlet):
             self._pool_spawn('store', self._retry_later, id, envelope, reply)
             raise
         else:
-            self._pool_spawn('store', self._remove, id)
+            if isinstance(results, collections.Mapping):
+                self._handle_partial_relay(id, envelope, attempts, results)
+            elif isinstance(results, collections.Sequence):
+                results = dict(zip(envelope.recipients, results))
+                self._handle_partial_relay(id, envelope, attempts, results)
+            else:
+                self._remove(id)
+
+    def _handle_partial_relay(self, id, envelope, attempts, results):
+        delivered = set()
+        tempfails = []
+        permfails = []
+        for rcpt, rcpt_res in results.items():
+            if rcpt_res is None or isinstance(rcpt_res, Reply):
+                delivered.add(envelope.recipients.index(rcpt))
+            elif isinstance(rcpt_res, PermanentRelayError):
+                delivered.add(envelope.recipients.index(rcpt))
+                permfails.append((rcpt, rcpt_res.reply))
+            elif isinstance(rcpt_res, TransientRelayError):
+                tempfails.append((rcpt, rcpt_res.reply))
+        if permfails:
+            rcpts, replies = zip(*permfails)
+            fail_env = envelope.copy(rcpts)
+            for reply, group_env in self._split_by_reply(fail_env, replies):
+                self._perm_fail(None, group_env, reply)
+        if tempfails:
+            rcpts, replies = zip(*tempfails)
+            fail_env = envelope.copy(rcpts)
+            if not self._retry_later(id, fail_env, replies):
+                return
+        else:
+            self.store.remove(id)
+            return
+        self.store.set_recipients_delivered(id, delivered)
 
     def _dequeue(self, id):
         try:
