@@ -19,26 +19,34 @@
 # THE SOFTWARE.
 #
 
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals
 
 import re
-import cStringIO
 from socket import error as socket_error
 from errno import ECONNRESET, EPIPE
+from io import BytesIO
 
 from gevent.ssl import SSLSocket, SSLError
-from gevent import socket
+import six
 
 from slimta import logging
+from slimta.util.typecheck import check_argtype
+from slimta.util.encoders import printable_decode, strict_encode
 from . import ConnectionLost, BadReply
-from .reply import Reply
+
+try:
+    from gevent.ssl import SSLWantReadError
+except ImportError:
+    # Some supported versions of gevent won't have this.
+    class SSLWantReadError(Exception):
+        pass
 
 __all__ = ['IO']
 
-line_pattern = re.compile(r'(.*?)\r?\n')
-reply_line_pattern = re.compile(r'((\d\d\d)([ \t-])(.*?))\r?\n')
-command_pattern = re.compile(r'^([a-zA-Z]+)\s*$')
-command_arg_pattern = re.compile(r'^([a-zA-Z]+)\s+(.+?)\s*$')
+line_pattern = re.compile(br'(.*?)\r?\n')
+reply_line_pattern = re.compile(br'((\d\d\d)([ \t-])(.*?))\r?\n')
+command_pattern = re.compile(br'^([a-zA-Z]+)\s*$')
+command_arg_pattern = re.compile(br'^([a-zA-Z]+)\s+(.+?)\s*$')
 
 log = logging.getSocketLogger(__name__)
 
@@ -50,8 +58,8 @@ class IO(object):
         if tls_wrapper:
             self._tls_wrapper = tls_wrapper
 
-        self.send_buffer = cStringIO.StringIO()
-        self.recv_buffer = ''
+        self.send_buffer = BytesIO()
+        self.recv_buffer = b''
 
     @property
     def encrypted(self):
@@ -62,16 +70,19 @@ class IO(object):
         if self.encrypted:
             try:
                 self.socket.unwrap()
-            except socket_error as (errno, message):
-                if errno not in (0, EPIPE, ECONNRESET):
+            except SSLWantReadError:
+                pass
+            except socket_error as e:
+                if e.errno not in (0, EPIPE, ECONNRESET):
                     raise
         self.socket.close()
 
     def raw_send(self, data):
+        check_argtype(data, six.binary_type, 'data')
         try:
             self.socket.sendall(data)
-        except socket_error as (errno, message):
-            if errno == ECONNRESET:
+        except socket_error as e:
+            if e.errno == ECONNRESET:
                 raise ConnectionLost()
             raise
         log.send(self.socket, data)
@@ -79,12 +90,12 @@ class IO(object):
     def raw_recv(self):
         try:
             data = self.socket.recv(4096)
-        except socket_error as (errno, message):
-            if errno == ECONNRESET:
+        except socket_error as e:
+            if e.errno == ECONNRESET:
                 raise ConnectionLost()
             raise
         log.recv(self.socket, data)
-        if data == '':
+        if data == b'':
             raise ConnectionLost()
         return data
 
@@ -110,10 +121,10 @@ class IO(object):
 
     def flush_send(self):
         send = self.send_buffer.getvalue()
-        if send == '':
+        if send == b'':
             return
         self.raw_send(send)
-        self.send_buffer = cStringIO.StringIO()
+        self.send_buffer = BytesIO()
 
     def recv_reply(self):
         code = None
@@ -132,7 +143,7 @@ class IO(object):
                     message_lines.append(match.group(4))
                     self.recv_buffer = input[match.end(0):]
 
-                    if match.group(3) != '-':
+                    if match.group(3) != b'-':
                         incomplete = False
                         start_i = None
                     else:
@@ -142,15 +153,17 @@ class IO(object):
                     if match:
                         self.recv_buffer = input[match.end(0):]
                         message_lines.append(match.group(1))
-                        raise BadReply('\r\n'.join(message_lines))
+                        raise BadReply(
+                            printable_decode(b'\r\n'.join(message_lines)))
                     else:
                         start_i = None
 
             if incomplete:
                 self.buffered_recv()
                 input = self.recv_buffer
+            body = b'\r\n'.join(message_lines)
 
-        return code, '\r\n'.join(message_lines)
+        return printable_decode(code), printable_decode(body)
 
     def recv_line(self):
         while True:
@@ -163,29 +176,34 @@ class IO(object):
 
     def recv_command(self):
         line = self.recv_line()
-        match = command_pattern.match(line)
-        if match:
-            return match.group(1).upper(), None
-        match = command_arg_pattern.match(line)
-        if match:
-            return match.group(1).upper(), match.group(2)
+        cmd_match = command_pattern.match(line)
+
+        if cmd_match:
+            return printable_decode(cmd_match.group(1).upper()), None
+        cmd_arg_match = command_arg_pattern.match(line)
+        if cmd_arg_match:
+            return (
+                printable_decode(cmd_arg_match.group(1).upper()),
+                printable_decode(cmd_arg_match.group(2)))
+
         return None, None
 
     def send_reply(self, reply):
-        code, message = reply.code, reply.message
+        code = strict_encode(reply.code)
+        message = strict_encode(reply.message)
         lines = []
-        message = message+'\r\n'
+        message = message+b'\r\n'
         for match in line_pattern.finditer(message):
             lines.append(match.group(1))
 
-        to_send = cStringIO.StringIO()
+        to_send = BytesIO()
         for line in lines[:-1]:
-            to_send.write(''.join((code, '-', line, '\r\n')))
-        to_send.write(''.join((code, ' ', lines[-1], '\r\n')))
+            to_send.write(b''.join((code, b'-', line, b'\r\n')))
+        to_send.write(b''.join((code, b' ', lines[-1], b'\r\n')))
         return self.buffered_send(to_send.getvalue())
 
     def send_command(self, command):
-        return self.buffered_send(command+'\r\n')
+        return self.buffered_send(strict_encode(command)+b'\r\n')
 
 
 # vim:et:fdm=marker:sts=4:sw=4:ts=4
