@@ -1,6 +1,7 @@
+from __future__ import unicode_literals
 
 import unittest2 as unittest
-from mox import MoxTestBase, IsA
+from mox3.mox import MoxTestBase, IsA
 import gevent
 from gevent.pywsgi import WSGIServer as GeventWSGIServer
 
