@@ -1,6 +1,7 @@
+from __future__ import unicode_literals
 
 import unittest2 as unittest
-from mox import MoxTestBase, IsA
+from mox3.mox import MoxTestBase, IsA
 
 from slimta.relay import Relay, PermanentRelayError, TransientRelayError
 from slimta.policy import RelayPolicy
