@@ -1,11 +1,14 @@
+from __future__ import unicode_literals
 
 import unittest2 as unittest
-from mox import MoxTestBase, IsA, IgnoreArg
+from mox3.mox import MoxTestBase, IsA, IgnoreArg
 from gevent.event import AsyncResult
 from gevent import Timeout
+from six.moves import urllib_parse
 
 from slimta.envelope import Envelope
 from slimta.util.deque import BlockingDeque
+from slimta.smtp.reply import Reply
 from slimta.relay import PermanentRelayError, TransientRelayError
 from slimta.relay.http import HttpRelay, HttpRelayClient
 from slimta.http import HTTPConnection
@@ -27,7 +30,7 @@ class TestHttpRelayClient(unittest.TestCase, MoxTestBase):
         class FakeRelay(object):
             queue = self.queue
             idle_timeout = None
-            url = 'http://testurl:8025/path/info'
+            url = urllib_parse.urlsplit('http://testurl:8025/path/info')
             tls = None
             http_verb = 'POST'
             sender_header = 'X-Envelope-Sender'
@@ -39,7 +42,7 @@ class TestHttpRelayClient(unittest.TestCase, MoxTestBase):
         self.client = HttpRelayClient(FakeRelay())
         self.result = self.mox.CreateMock(AsyncResult)
         self.env = Envelope('sender@example.com', ['rcpt1@example.com', 'rcpt2@example.com'])
-        self.env.parse('Header: value\r\n\r\ntest message\r\n')
+        self.env.parse(b'Header: value\r\n\r\ntest message\r\n')
 
     def test_wait_for_request(self):
         self.mox.StubOutWithMock(self.client, '_handle_request')
@@ -59,15 +62,16 @@ class TestHttpRelayClient(unittest.TestCase, MoxTestBase):
     def test_handle_request(self):
         self.mox.StubOutWithMock(self.client, '_process_response')
         conn = self.client.conn = self.mox.CreateMockAnything()
+        self.client.ehlo_as = 'test'
         conn.putrequest('POST', '/path/info')
-        conn.putheader('Content-Length', 31)
-        conn.putheader('Content-Type', 'message/rfc822')
-        conn.putheader('X-Ehlo', 'test')
-        conn.putheader('X-Envelope-Sender', 'c2VuZGVyQGV4YW1wbGUuY29t')
-        conn.putheader('X-Envelope-Recipient', 'cmNwdDFAZXhhbXBsZS5jb20=')
-        conn.putheader('X-Envelope-Recipient', 'cmNwdDJAZXhhbXBsZS5jb20=')
-        conn.endheaders('Header: value\r\n\r\n')
-        conn.send('test message\r\n')
+        conn.putheader(b'Content-Length', 31)
+        conn.putheader(b'Content-Type', b'message/rfc822')
+        conn.putheader(b'X-Ehlo', b'test')
+        conn.putheader(b'X-Envelope-Sender', b'c2VuZGVyQGV4YW1wbGUuY29t')
+        conn.putheader(b'X-Envelope-Recipient', b'cmNwdDFAZXhhbXBsZS5jb20=')
+        conn.putheader(b'X-Envelope-Recipient', b'cmNwdDJAZXhhbXBsZS5jb20=')
+        conn.endheaders(b'Header: value\r\n\r\n')
+        conn.send(b'test message\r\n')
         conn.getresponse().AndReturn(13)
         self.client._process_response(13, 21)
         self.mox.ReplayAll()
@@ -96,7 +100,7 @@ class TestHttpRelayClient(unittest.TestCase, MoxTestBase):
         http_res.reason = 'OK'
         http_res.getheader('X-Smtp-Reply', '').AndReturn('250; message="2.0.0 Ok"')
         http_res.getheaders()
-        self.result.set(True)
+        self.result.set(Reply('250', '2.0.0 Ok'))
         self.mox.ReplayAll()
         self.client._process_response(http_res, self.result)
 
